%% Apparent exchange rate imaging in anisotropic systems
% This script loads the diffusion imaging data and calculates the
% AXR and sigma values along the pricipal directions of the diffusions
% tensor D. See Snderby et. al. 2013 or http://dig.drcmr.dk/axr-dataset/ 
% for further details
%
% Snderby, C. K., Lundell, H. M., Sgaard, L. V. and Dyrby, T. B. (2013), 
% Apparent exchange rate imaging in anisotropic systems. 
% Magn Reson Med. doi: 10.1002/mrm.24957
%
%
% Third party tools needed:
% Tools for NIfTI and ANALYZE image  
% http://www.mathworks.com/matlabcentral/fileexchange/8797-tools-for-nifti-and-analyze-image
% 
% ImageMagic (optional)
% http://www.imagemagick.org/
%
% 
% Diffusion Imaging Group (DIG)
% Danish Research Centre for Magnetic Resonance, 
% Centre for Functional and Diagnostic Imaging and Research, 
% Copenhagen University Hospital Hvidovre, Hvidovre, Denmark.
%
%
%% START OF SCRIPT
close all
clear all
load data/ROI;
addpath NIFTI; 
addpath helper_scripts/
mkdir figures

dir = [0 0 0 -0.9479 -0.3997 -0.2247 -0.6257 0.6646 -0.8549 -0.2561 -0.1984 0.1683 -0.5056 0.6902 -0.0028 -0.9662 -0.4288 0.4909 0.1097 0.7012 -0.8622 -0.6886 0.2664;... 
       0 0 0 0.0333 -0.6809 0.6351 -0.2239 0.1129 -0.4801 0.0837 0.1832 0.8965 -0.8609 -0.4039 -0.9806 -0.0477 0.867 -0.8117 -0.6558 0.5835 0.5064 0.4059 0.453;...
       0 0 0 0.3166 0.6135 0.7389 0.7471 0.7385 0.1959 0.9629 -0.9627 0.4096 0.0553 0.6002 0.1955 -0.2528 0.2534 0.316 0.7468 0.4094 0.0003 0.6007 0.8506]';
   
dir_sphere = [dir; -dir]; 
bval = 3.0509e+03;
tm = [8 57 106 145 203 251 300]; %ms

%% LOAD THE DATA AND CALCULATE THE D and D' tensors
sig_adcm_nii = load_nii('data/sig_adcm');
sig0_adcm_nii = load_nii('data/sig0_adcm');
sig_adc_nii = load_nii('data/sig_adc');
sig0_adc_nii = load_nii('data/sig0_adc');


%calulate the D'(tm) tensors
Dmtensor = zeros(6,128,64,7);

for i = 1:7
    sig_adcm = sig_adcm_nii.img(:,:,:,i);
    sig0_adcm = sig0_adcm_nii.img(:,:,:,i);
    Dmtensor(:,:,:,i) = calctensorfromsig(sig_adcm,sig0_adcm,bval,dir(4:end,:),BRAIN_ROI);
end

%Calculate D tensor
sig_adc = sig_adc_nii.img;
sig0_adc = sig0_adc_nii.img;
Dtensor = calctensorfromsig(sig_adc,sig0_adc,bval,dir(4:end,:),BRAIN_ROI);

%% Calculate the Eigen Values and Eigenvectors
Dmtensor = vector2tensor(Dmtensor);
[D_eigvec, D_eigval] = eig_matrix(vector2tensor(Dtensor));
[Dm_eigvec, Dm_eigval] = eig_matrix(Dmtensor);

%calculates Dm values parallel to the eigenvectors of D - eg projetcs
%Dm onto the eigenvectors of D
Dmtensor_par = zeros(3,3,128,64,7);
for x = 1:128;
    for y = 1:64
        v1 = D_eigvec(:,1,x,y);
        v2 = D_eigvec(:,2,x,y);
        v3 = D_eigvec(:,3,x,y);
        for t = 1:7
            Dm = Dmtensor(:,:,x,y,t);
            Dmtensor_par(1,1,x,y,t) = v1'*Dm*v1;
            Dmtensor_par(2,2,x,y,t) = v2'*Dm*v2;
            Dmtensor_par(3,3,x,y,t) = v3'*Dm*v3;
        end
    end
end

%% Fits to eigenvalues units changed from [mm^2/s ->  mm^2/ms]  
% This might take a while...
[AXR, SIGMA, rsquare, SE_AXR, SE_SIGMA] = fit_eigenval(Dmtensor_par*1e3, D_eigval*1e3, tm*1e-3, BRAIN_ROI);

% If you can't get the fitting procedure to work uncomment the following line
% load fitting_data;

%% CALC FA and MD MAPS.
FA_D_eigval = calcFA(D_eigval);
MD_D_eigval = calcMD(D_eigval);

%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% PLOT SOME OF THE RESULTS %%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% PLOT FIGURE 4 from the paper
scale_h = (64-31)/(47-21);
pos = [0.025 0.03 0.95 0.90];
pos2 = pos;
width_image = 10;
width_colorbar_img = 14;
pos2(3) = pos2(3)*10/width_colorbar_img;
title_pos = [0.5,1 0];
fontsize = 24;
figure(1); imagesc(squeeze(AXR(3,3,31:64,21:47))); axis image; axis off; set(gca,'position',pos,'units','normalized'); caxis([0 15])
%h = title('a)'); set(h,'Position',title_pos,'units','normalized')
h=title('a)'); p=get(h,'Position'); p(2)=p(2)+0.25; set(h,'Position',p)
save_fig('figures/FIG4_AXR1',[],[width_image width_image*scale_h],fontsize)
figure(2); imagesc(squeeze(AXR(2,2,31:64,21:47))); axis image; axis off; set(gca,'position',pos,'units','normalized'); caxis([0 15])
%h = title('b)'); set(h,'Position',title_pos,'units','normalized')
h=title('b)'); set(h,'Position',p)
save_fig('figures/FIG4_AXR2',[],[width_image width_image*scale_h],fontsize)
figure(3); imagesc(squeeze(AXR(1,1,31:64,21:47))); axis image; axis off; set(gca,'position',pos,'units','normalized'); caxis([0 15])
%h = title('c)'); set(h,'Position',title_pos,'units','normalized')
h=title('c)'); set(h,'Position',p)
save_fig('figures/FIG4_AXR3',[],[width_image width_image*scale_h],fontsize)
imagesc(squeeze(AXR(1,1,31:64,21:47))); axis image; axis off; caxis([0 15]); cb_h=colorbar; set(gca,'position',pos2 ,'units','normalized');
%h = title('c)'); set(h,'Position',title_pos,'units','normalized')
%set(get(cb_h,'ylabel'),'string','AXR [s^{-1}]')
text('String','AXR [s^{-1}]','Rotation',270,'Position',[36.7 13.9840909090909 217.640507506085],'FontSize',24);
h=title('c)');set(h,'Position',p)
save_fig('figures/FIG4_AXR3_colorbar',[],[width_colorbar_img 10*scale_h],fontsize)

%plot sigma
figure(4); imagesc(squeeze(SIGMA(3,3,31:64,21:47))); axis image; axis off; set(gca,'position',pos,'units','normalized'); colormap gray; caxis([0 0.5])
h=title('e)'); set(h,'Position',p)
save_fig('figures/FIG4_SIGMA1',[],[width_image width_image*scale_h],fontsize)
%h = title('e)'); set(h,'Position',title_pos,'units','normalized')
figure(5); imagesc(squeeze(SIGMA(2,2,31:64,21:47))); axis image; axis off; set(gca,'position',pos,'units','normalized'); colormap gray; caxis([0 0.5])
%h = title('f)'); set(h,'Position',title_pos,'units','normalized')
h=title('f)'); set(h,'Position',p)
save_fig('figures/FIG4_SIGMA2',[],[width_image width_image*scale_h],fontsize)
figure(6); imagesc(squeeze(SIGMA(1,1,31:64,21:47))); axis image; axis off; set(gca,'position',pos,'units','normalized'); colormap gray; caxis([0 0.5])
%h = title('g)'); set(h,'Position',title_pos,'units','normalized')
h=title('g)'); set(h,'Position',p)
save_fig('figures/FIG4_SIGMA3',[],[width_image width_image*scale_h],fontsize)
imagesc(squeeze(SIGMA(1,1,31:64,21:47))); axis image; axis off; colormap gray; caxis([0 0.5]); cb_h=colorbar; set(gca,'position',pos2 ,'units','normalized');
%h = title('g)'); set(h,'Position',title_pos,'units','normalized')
%set(get(cb_h,'ylabel'),'string','$\sigma$','interpreter','latex')
%set(get(cb_h,'ylabel'),'string','sigma')
text('String','$\sigma$','interpreter','latex','Rotation',270,'Position',[36.7 17.2295454545455 217.640507506085],'FontSize',24);
h=title('g)'); set(h,'Position',p)
save_fig('figures/FIG4_SIGMA3_colorbar',[],[width_colorbar_img 10*scale_h],fontsize)


%plot FA map eigvals
figure(7); imagesc(squeeze(FA_D_eigval(31:64,21:47))); axis image; axis off; set(gca,'position',pos,'units','normalized'); colormap gray; caxis([0 0.7])
h=title('h)'); set(h,'Position',p)
save_fig('figures/FIG4_FA_D',[],[width_image width_image*scale_h],fontsize)
imagesc(squeeze(FA_D_eigval(31:64,21:47))); axis image; axis off; colormap gray; caxis([0 0.7]); cb_h=colorbar; set(gca,'position',pos2 ,'units','normalized'); 
%h = title('d)FA D'); set(h,'Position',title_pos,'units','normalized')
%set(get(cb_h,'ylabel'),'string','FA')
text('String','FA','Rotation',270,'Position',[37 16.3022727272727 217.640507506085],'FontSize',24);
h=title('h)'); set(h,'Position',p)
save_fig('figures/FIG4_FA_D_colorbar',[],[width_colorbar_img 10*scale_h],fontsize)

%plot MD
figure(8); imagesc(squeeze(MD_D_eigval(31:64,21:47))*1e3); axis image; axis off; set(gca,'position',pos,'units','normalized'); 
colormap gray; caxis([0,0.45])
h=title('d)'); set(h,'Position',p)
save_fig('figures/FIG4_MD_D',[],[width_image width_image*scale_h],fontsize)
imagesc(squeeze(MD_D_eigval(31:64,21:47))*1e3); axis image; axis off; colormap gray; cb_h = colorbar; caxis([0,0.45]); set(gca,'position',pos2 ,'units','normalized'); 
%h = title('h) MD D'); set(h,'Position',title_pos,'units','normalized')
%set(get(cb_h,'ylabel'),'string','MD [\mum^2/ms]'), %'Rotation',-90
text('String','MD [\mum^2/ms]','Rotation',270,'Position',[37 12.2840909090909 217.640507506085],'FontSize',24);
h=title('d)'); set(h,'Position',p)
save_fig('figures/FIG4_MD_D_colorbar',[],[width_colorbar_img 10*scale_h],fontsize)



%use ImageMagick to merge the PNG files using the convert command 
% (http://www.imagemagick.org/script/binary-releases.php ) 

figure_path = [pwd '/figures/'];
ImageMagic_bin = '/usr/local/bin/'; %change to appropriate folder

cmd1 = [ImageMagic_bin 'convert +append '...
    figure_path 'FIG4_AXR1.png '...
    figure_path 'FIG4_AXR2.png '...
    figure_path 'FIG4_AXR3_colorbar.png ' ...
    figure_path 'FIG4_MD_D_colorbar.png ' ... 
    figure_path 'merged_top.png'];

cmd2 = [ImageMagic_bin 'convert +append '...
    figure_path 'FIG4_SIGMA1.png '...
    figure_path 'FIG4_SIGMA2.png '...
    figure_path 'FIG4_SIGMA3_colorbar.png ' ...
    figure_path 'FIG4_FA_D_colorbar.png ' ... 
    figure_path 'merged_bottom.png'];

cmd3 = [ImageMagic_bin 'convert -append '...
     figure_path 'merged_top.png '...
     figure_path 'merged_bottom.png '...
     figure_path 'FIG4_maps.png'];
system(cmd1)
system(cmd2)
system(cmd3)

delete ./figures/merged_top.png
delete ./figures/merged_bottom.png

%% Plot eigenvalues + plus fit for single voxel
x = 50;
y = 29;
tmvec = 1:400; %[ms]


sigma =            diag(SIGMA(:,:,x,y));
axr =                diag(AXR(:,:,x,y));
D =                diag(D_eigval(:,:,x,y));
Dm =      [squeeze(Dmtensor_par(1,1,x,y,:))';...
             squeeze(Dmtensor_par(2,2,x,y,:))';...
             squeeze(Dmtensor_par(3,3,x,y,:))'];

Dm_model = AXR_model(axr*1e-3,sigma,D*1e3,tmvec);

figure(9)
plot(tmvec,Dm_model,'--')
hold on
plot(tm,Dm*1e3,'x','MarkerSize',15)
plot(380,D*1e3,'*','MarkerSize',15)
xlabel('t_m [ms]'); ylabel('D''[\mum^2/ms]')
legend('\lambda_1','\lambda_2','\lambda_3','Location','NorthWest'); legend boxoff;
title(['sigma= ' num2str(sigma(1)) '/'...
                 num2str(sigma(2)) '/'...
                 num2str(sigma(3)) '/'...                 
       ' AXR= '  num2str(axr(1)) '/'...
                 num2str(axr(2)) '/'...
                 num2str(axr(3))])
save_fig('figures/FIG2_Dm(tm)_vs_tm',[],[10,10],[])


%% FIGURE 3
scale_h = (64-31)/(47-21);
pos = [0.1 0.1 0.9 0.77];
pos2 = [0.125 0.1 1 0.77];
width_image = 10;
width_colorbar_img = 14;
pos2(3) = pos2(3)*10/width_colorbar_img;
title_pos = [0.5,1 0];
fontsize = 24;
xp = [50];
yp = [29];

figure(10); hold on
xlim([0.5 3.5])
ylim([0.8 12])
h = errorbar(1,AXR(3,3,xp,yp), SE_AXR(3,3,xp,yp),'r','Marker','o'); errorbar_tick(h,20); hold on; set(gca,'position',pos2,'units','normalized');
h = errorbar(2,AXR(2,2,xp,yp), SE_AXR(2,2,xp,yp),'g','Marker','o'); errorbar_tick(h,20);
h = errorbar(3,AXR(1,1,xp,yp), SE_AXR(1,1,xp,yp),'b','Marker','o'); errorbar_tick(h,20);
%set(gca,'XTickLabel',{'$v_1$','v_2','v_3'},'XTick',[1 2 3]);
format_ticks(gca,{'$AXR_1$','$AXR_2$','$AXR_3$'},{'$2$','$4$','$6$','$8$','$10$', '$12$'},[1,2,3],[2 4 6 8 10 12]); set(gca,'FontSize',fontsize); obj = findall(gcf,'type','Text'); set(obj,'FontSize',fontsize)
%set(gca,'YTickLabel',{'0.1','0.2','0.3' '0.4'},'YTick',[0.1 0.2 0.3 0.4]);
ylim([0.8 12])
%ylim([0.05 0.42])
title('AXR','interpreter','latex','FontSize',34);
%ylabel({'$[s^{-1}]$',' '},'interpreter','latex'); %ylabel('Y'); zlabel('Z');

save_fig('figures/FIG3_axr',3,[width_colorbar_img*0.8 10*scale_h],-1)
hold off; clf

%SIGMA
figure(11); hold on
xlim([0.5 3.5])
ylim([0.05 0.5])
h = errorbar(1,SIGMA(3,3,xp,yp), SE_SIGMA(3,3,xp,yp),'r','Marker','o'); errorbar_tick(h,20); hold on; set(gca,'position',pos2,'units','normalized');
h = errorbar(2,SIGMA(2,2,xp,yp), SE_SIGMA(2,2,xp,yp),'g','Marker','o'); errorbar_tick(h,20);
h = errorbar(3,SIGMA(1,1,xp,yp), SE_SIGMA(1,1,xp,yp),'b','Marker','o'); errorbar_tick(h,20);
%set(gca,'XTickLabel',{'$v_1$','v_2','v_3'},'XTick',[1 2 3]);
format_ticks(gca,{'$\sigma_1$','$\sigma_2$','$\sigma_3$'},{'$0.1$','$0.2$','$0.3$','$0.4$','$0.5$'},[1,2,3],[0.1 0.2 0.3 0.4 0.5]); set(gca,'FontSize',fontsize); obj = findall(gcf,'type','Text'); set(obj,'FontSize',fontsize)
%set(gca,'YTickLabel',{'0.1','0.2','0.3' '0.4'},'YTick',[0.1 0.2 0.3 0.4]);
xlim([0.5 3.5])
ylim([0.05 0.5])
title('$\sigma$','interpreter','latex','FontSize',34);
save_fig('figures/FIG3_sigma',3,[width_colorbar_img*0.8 10*scale_h],-1)
hold off; clf

% D
figure(12);
V = D_eigvec(:,:,xp,yp);
L = D_eigval(:,:,xp,yp)*1e3;
[X,Y,Z]=ellipsoid(0,0,0,L(1,1),L(2,2),L(3,3),20);

sz=size(X);
for x=1:sz(1)
    for y=1:sz(2)
        A=[X(x,y) Y(x,y) Z(x,y)]';
        A=V*A; %added the /delta to reduce the sizes of the ellipsoids without changing the axis'
        X(x,y)=A(1);Y(x,y)=A(2);Z(x,y)=A(3);
    end
end
h = surf(X,Y,Z,'LineWidth',2); axis equal; set(gca,'position',pos,'units','normalized');
alpha(h,0.0)

hold on
scale = 1.3;
quiver3(0,0,0,V(1,3)*scale*L(3,3),V(2,3)*scale*L(3,3),V(3,3)*scale*L(3,3),'r','LineWidth',3); set(gca,'FontSize',fontsize); obj = findall(gcf,'type','Text'); set(obj,'FontSize',fontsize)
quiver3(0,0,0,V(1,2)*scale*L(2,2),V(2,2)*scale*L(2,2),V(3,2)*scale*L(2,2),'g','LineWidth',3); set(gca,'FontName','Latin Modern Roman');
quiver3(0,0,0,V(1,1)*scale*L(1,1),V(2,1)*scale*L(1,1),V(3,1)*scale*L(1,1),'b','LineWidth',3)
title('D','interpreter','latex','FontSize',34);
xlabel('$[\mu m^2/ms]$','interpreter','latex'); %ylabel('Y'); zlabel('Z');
save_fig('figures/FIG3_Dtensor',3,[width_colorbar_img*0.9 10*scale_h],-1)
hold off

%use ImageMagick to merge the PNG files using the convert command 
% (http://www.imagemagick.org/script/binary-releases.php ) 
figure_path = [pwd '/figures/'];
ImageMagic_bin = '/usr/local/bin/'; %change to appropriate folder
cmd1 = [ImageMagic_bin 'convert +append '...
    figure_path 'FIG3_Dtensor.png '...
    figure_path 'FIG3_axr.png '...
    figure_path 'FIG3_sigma.png '...
    figure_path 'FIG3_D_axr_sigma.png'];

system(cmd1)

